# UI for picking material, typically used for web dialogs.
# Behavior inspired by Layout color or pattern selection.
#
# NOTE: When loading this example from console, not on Sketchup start, a model
# must be opened or a new model created for observer to be initialized.
#
# To test dialog, run "MaterialSelectorTest.example".
# ... or load "ene_pickmaterial.rb" (can be put in root "Plugins" folder,)
# and use test global method epm() from the console.


module MaterialSelectorTest

  WIN =( Sketchup.platform == :platform_win rescue RUBY_PLATFORM !~ /darwin/i )
  
  VERSION = '0.2.0' # This version was modified from Eneroth's online example.

  @@loaded = false unless defined?(@@loaded)
  if !@@loaded
    @@dialog = nil   # ref for the material picker WebDialog
    @@app_observer = nil  # ref for the AppObserver
    # ... etc.
  end

  # Interface for picking material from material browser.
  class MaterialSelector

    # Block to call when a material is selected.
    # TODO: store blocks as hash indexed by calling Model to function better in multi document view?
    @@callback = nil

    # Public: Open the material browser to let user choose a material.
    # Typically called when user clicks a button for making material input in
    # web dialog.
    #
    # old_material - Material to be selected when material browser starts
    #                or nil to not select any specific material in browser
    #                (default: nil)
    #
    # Yields Material whenever a material is selected.
    #
    # Returns nothing.
    def self.pick_material(old_material = nil, &callback)
      # Disable callback if any so current material
      #  first can be set to old_material.
      @@callback = nil
      Sketchup.active_model.materials.current = old_material if old_material
      self.open_material_inspector
      # Selecting any material will activate the Paint Bucket tool.
      @@callback = callback
      nil
    end ###
    
    # Public: Stop executing callback when materials are selected.
    # Typically called when the web dialog containing the material selection
    # input is closed.
    #
    # returns nothing.
    def self.done_picking_material
      @@callback = nil
      
      # REVIEW: If possible, maybe reset material inspector
      # view state to what it was before calling select_material.
      Sketchup.send_action("selectSelectionTool:")
      nil
    end ###
    
    # Internal: Open the material inspector.
    def self.open_material_inspector
      # TODO: Look up how these function on MAC.
      # TODO: If material browser is already opened it should somehow be focused to attract the user's attention.
      if Sketchup.version.to_i >= 16
        # In 2015 this toggles the material browser instead of showing it.
        UI.show_inspector "Materials"
      elsif WIN
        # Windows only for opening the inspector.
        Sketchup.send_action(21074)
      else
        # Select paint bucket tool.
        # Has the drawback that the tool is selected.
        Sketchup.send_action("selectPaintTool:")
      end
      #
      # Selecting any material will activate the Paint Bucket tool, anyway.
      #
    end ###
    
    # Observer interface.  
    
    # Internal: Current material has changed.
    def self.onMaterialSetCurrent(clicked_material)
      @@callback.call(clicked_material) if @@callback
      Sketchup.send_action("selectSelectionTool:")
    end ###
    
    # TODO: If such an observer is ever implemented, implement this.
    # Internal: Material inspector was closed
    def self.onClosingMaterialInspector
      @@callback = nil
    end ###

  end # class MaterialSelector


  # Example web dialog.
  def self.example
    
    if @dlg && @dlg.is_a?(UI::WebDialog)
      if @dlg.visible?
        @dlg.bring_to_front
      else
        WIN ? @dlg.show : @dlg.show_modal
      end
      return UI.beep
    end
    
    @dlg = UI::WebDialog.new(
      "Pick Material Example",
      false,
      "ene_pick_material",
      284,
      322,
      100,
      100,
      true
    )
    @dlg.navigation_buttons_enabled = false
    @dlg.set_background_color(@dlg.get_default_dialog_color)
    @dlg.set_file("#{File.dirname(__FILE__)}/dialog.html")
    WIN ? @dlg.show : @dlg.show_modal

    # Start picking material.
    @dlg.add_action_callback("pick_material") do |_, input_id|

      MaterialSelector.pick_material(nil) do |new_material|

        # Set material preview in dialog.
        # For this example only the color is used but texture could be saved to
        # file using write_thumbnail.
        if new_material
          color = new_material.color
          color_string = "rgb(#{color.red},#{color.green},#{color.blue})"
        else
          # When material is an image similar to the one inside Sketchup showing
          # front and back color could be used.
          color_string = "transparent"
        end
        js = "setPreview('#{input_id}', '#{color_string}');"
        @dlg.execute_script js

        # Save the given material reference and the relation to the given
        # input_id.
        puts "Material #{new_material} was selected for input #{input_id}."

      end

    end

    # Stop picking material.
    @dlg.set_on_close do
      MaterialSelector.done_picking_material
    end

  end


  # Observer stuff.
  class MyMaterialsObserver < Sketchup::MaterialsObserver

    def onMaterialSetCurrent(_, material)
      MaterialSelector.onMaterialSetCurrent material
    end

  end # class MyMaterialsObserver

  class MyAppObserver < Sketchup::AppObserver

    def expectsStartupModelNotifications    
      true     
    end
    
    def onNewModel(model)    
      model.materials.add_observer MyMaterialsObserver.new      
    end
    
    def onOpenModel(model)    
      model.materials.add_observer MyMaterialsObserver.new      
    end

  end # class MyAppObserver


  unless @@app_observer
    @@app_observer = MyAppObserver.new
    Sketchup.add_observer @@app_observer
    @@loaded = true
  end

end # plugin module